/*
 * Copyright 2000-2017 Vaadin Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package de.mcs.tools.sps.ui;

import com.vaadin.flow.component.Text;
import com.vaadin.flow.component.dependency.HtmlImport;
import com.vaadin.flow.component.html.Div;
import com.vaadin.flow.component.html.H2;
import com.vaadin.flow.component.icon.Icon;
import com.vaadin.flow.component.icon.VaadinIcon;
import com.vaadin.flow.component.page.Viewport;
import com.vaadin.flow.router.HighlightConditions;
import com.vaadin.flow.router.RouterLayout;
import com.vaadin.flow.router.RouterLink;
import com.vaadin.flow.server.InitialPageSettings;
import com.vaadin.flow.server.PWA;
import com.vaadin.flow.server.PageConfigurator;

import de.mcs.tools.sps.ui.views.assemblerview.AssemblerView;
import de.mcs.tools.sps.ui.views.emulatorview.EmulatorView;

/**
 * The main layout contains the header with the navigation buttons, and the
 * child views below that.
 */
@HtmlImport("frontend://styles/shared-styles.html")
@PWA(name = "ArduinoSPS/TinySPS", shortName = "ArduinoSPS")
@Viewport("width=device-width, minimum-scale=1.0, initial-scale=1.0, user-scalable=yes")
public class MainLayout extends Div implements RouterLayout, PageConfigurator {

  public MainLayout() {
    H2 title = new H2("ArduinoSPS/TinySPS");
    title.addClassName("main-layout__title");

    RouterLink reviews = new RouterLink(null, AssemblerView.class);
    reviews.add(new Icon(VaadinIcon.LINES), new Text("SPS Assembler"));
    reviews.addClassName("main-layout__nav-item");
    // Only show as active for the exact URL, but not for sub paths
    reviews.setHighlightCondition(HighlightConditions.sameLocation());

    RouterLink categories = new RouterLink(null, EmulatorView.class);
    categories.add(new Icon(VaadinIcon.COG), new Text("SPS Emulator"));
    categories.addClassName("main-layout__nav-item");

    Div navigation = new Div(reviews, categories);
    navigation.addClassName("main-layout__nav");

    Div header = new Div(title, navigation);
    header.addClassName("main-layout__header");
    add(header);

    addClassName("main-layout");
  }

  @Override
  public void configurePage(InitialPageSettings settings) {
    settings.addMetaTag("apple-mobile-web-app-capable", "yes");
    settings.addMetaTag("apple-mobile-web-app-status-bar-style", "black");
  }
}
