package de.mcs.tools.sps.emulator.model;

import java.util.Arrays;
import java.util.HashSet;
import java.util.Set;

public class CommandModel {

  public enum COMMAND {
    START, STOP, NEXT, RESET, COMPILE, NN
  };

  private Set<COMMAND> availableCommands;
  private COMMAND actualCommand;

  public CommandModel() {
    setActualCommand(COMMAND.NN);
    setAvailableCommands(new HashSet<>());
    evaluateAvailableCommands(getActualCommand());
  }

  @Override
  public String toString() {
    StringBuilder b = new StringBuilder();
    b.append(String.format("%s:[available: [%s], actual: %s]", this.getClass().getSimpleName(),
        Arrays.toString(availableCommands.toArray()), actualCommand.name()));
    return b.toString();
  }

  public void evaluateAvailableCommands(COMMAND command) {
    getAvailableCommands().clear();
    switch (command) {
    case START:
      getAvailableCommands().add(COMMAND.NEXT);
      getAvailableCommands().add(COMMAND.STOP);
      getAvailableCommands().add(COMMAND.RESET);
      break;
    case STOP:
      getAvailableCommands().add(COMMAND.START);
      getAvailableCommands().add(COMMAND.RESET);
      break;
    case NEXT:
      getAvailableCommands().add(COMMAND.STOP);
      getAvailableCommands().add(COMMAND.RESET);
      break;
    case COMPILE:
    case RESET:
    case NN:
      getAvailableCommands().add(COMMAND.START);
      getAvailableCommands().add(COMMAND.COMPILE);
      break;
    default:
      break;
    }
  }

  /**
   * @return the availableCommands
   */
  public Set<COMMAND> getAvailableCommands() {
    return availableCommands;
  }

  /**
   * @param availableCommands
   *          the availableCommands to set
   */
  public void setAvailableCommands(Set<COMMAND> availableCommands) {
    this.availableCommands = availableCommands;
  }

  /**
   * @return the actualCommand
   */
  public COMMAND getActualCommand() {
    return actualCommand;
  }

  /**
   * @param actualCommand
   *          the actualCommand to set
   */
  public void setActualCommand(COMMAND actualCommand) {
    this.actualCommand = actualCommand;
  }
}
