package de.mcs.tools.sps.emulator.model;

import java.io.UnsupportedEncodingException;
import java.util.Arrays;

import org.apache.commons.lang3.StringUtils;

import de.mcs.utils.SHA256Utils;

public class ProgramModel {

  private Hardware hardware;
  private String[] source;
  private String hash;
  private byte[] bin;
  private boolean modified;
  private int actualLine;
  private int[] bin2SrcLine;

  public ProgramModel() {
    hardware = Hardware.HOLTEK;
    modified = false;
    setActualLine(0);
  }

  @Override
  public String toString() {
    StringBuilder b = new StringBuilder();
    b.append(String.format("%s:[Hardware: %s, Hash: %s, modified: %s, actualLine: %d, Source: \"%s\", bin: 0x%x ]",
        this.getClass().getSimpleName(), hardware.name(), hash, modified, actualLine,
        (source != null) ? source[0] + ((source.length > 1) ? "..." : "") : "", (bin != null) ? bin.length : 0));
    return b.toString();
  }

  /**
   * @return the source
   */
  public String[] getSource() {
    return source;
  }

  /**
   * @param source
   *          the source to set
   */
  public void setSource(String[] source) {
    this.source = source;
  }

  /**
   * @return the hash
   */
  public String getHash() {
    return hash;
  }

  /**
   * @param hash
   *          the hash to set
   */
  public void setHash(String hash) {
    this.hash = hash;
  }

  /**
   * @return the bin
   */
  public byte[] getBin() {
    return bin;
  }

  /**
   * @param bin
   *          the bin to set
   */
  public void setBin(byte[] bin) {
    this.bin = bin;
  }

  /**
   * @return the hardware
   */
  public Hardware getHardware() {
    return hardware;
  }

  /**
   * @param hardware
   *          the hardware to set
   */
  public void setHardware(Hardware hardware) {
    this.hardware = hardware;
  }

  /**
   * @return the modified
   */
  public boolean isModified() {
    return modified;
  }

  /**
   * @param modified
   *          the modified to set
   */
  public void setModified(boolean modified) {
    this.modified = modified;
  }

  /**
   * @return the actualLine
   */
  public int getActualLine() {
    return actualLine;
  }

  /**
   * @param actualLine
   *          the actualLine to set
   */
  public void setActualLine(int actualLine) {
    this.actualLine = actualLine;
  }

  public boolean rebuildHash() {
    if ((source != null) && (source.length > 0)) {
      String sourceStr = Arrays.toString(source);
      try {
        String newHash = SHA256Utils.shaBytesToString(SHA256Utils.shaBytes(sourceStr.getBytes("UTF-8")));
        if (StringUtils.isEmpty(hash)) {
          this.hash = newHash;
          this.setModified(false);
        } else {
          this.setModified(true);
        }
      } catch (UnsupportedEncodingException e) {
        e.printStackTrace();
      }
    }
    return isModified();
  }

  /**
   * @return the bin2SrcLine
   */
  public int[] getBin2SrcLine() {
    return bin2SrcLine;
  }

  /**
   * @param bin2SrcLine
   *          the bin2SrcLine to set
   */
  public void setBin2SrcLine(int[] bin2SrcLine) {
    this.bin2SrcLine = bin2SrcLine;
  }

}
