package de.mcs.utils;
/**
 * 
 */

import java.util.HashMap;
import java.util.Map;

import com.fasterxml.jackson.annotation.JsonAnyGetter;
import com.fasterxml.jackson.annotation.JsonAnySetter;
import com.fasterxml.jackson.annotation.JsonIgnore;

/**
 * @author w.klaas
 */
public class BaseApiModel {

  private Map<String, Object> values;

  /**
   * 
   */
  public BaseApiModel() {
    values = new HashMap<>();
  }

  /**
   * @param name
   *          name of the field
   * @param value
   *          value of the field
   */
  @JsonAnySetter
  public void setKeyValue(String name, Object value) {
    values.put(name, value);
  }

  /**
   * simple setting fuild values
   * 
   * @param name
   *          name of the field
   * @param value
   *          value of the field
   * @return BaseApiModel
   */
  @JsonIgnore
  public BaseApiModel set(String name, Object value) {
    values.put(name, value);
    return this;
  }

  /**
   * @return a map with all fields
   */
  @JsonAnyGetter
  public Map<String, Object> getAny() {
    return values;
  }

  /**
   * getting the value of a single field with the desired name
   * 
   * @param name
   *          the name of the field to get
   * @return value the value of the field, or <code>null</code>
   */
  public Object getField(String name) {
    return values.get(name);
  }

  /**
   * getting the value of a single field with the desired name
   * 
   * @param name
   *          the name of the field to get
   * @return value the value of the field as a string, or <code>null</code>
   */
  public String getFieldValueAsString(String name) {
    Object fieldValue = getField(name);
    if (fieldValue != null) {
      return fieldValue.toString();
    }
    return null;
  }

  @Override
  public String toString() {
    return values.toString();
  }

  /**
   * testing if an field is availble
   * 
   * @param name
   *          name of the field
   * @return <code>true</code> if the field is present
   */
  public boolean hasField(String name) {
    return values.containsKey(name);
  }

}
