package de.mcs.utils;

import static org.junit.Assert.fail;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.security.GeneralSecurityException;
import java.security.InvalidKeyException;
import java.security.Key;
import java.security.KeyStore;
import java.security.NoSuchAlgorithmException;
import java.security.PrivateKey;
import java.security.Signature;
import java.security.SignatureException;
import java.security.cert.CertificateException;
import java.security.cert.CertificateFactory;
import java.security.cert.X509Certificate;
import java.util.Base64;

public class SignUtils {

  private static final String KEYSTORE_PASSWORD = "easy1234";

  /**
   * signing a string with the private key from the keystore. (should be in the working directory, named keystore.jks)
   * 
   * @param string
   * @return the signature as a mimecoded string
   * @throws GeneralSecurityException
   * @throws FileNotFoundException
   * @throws IOException
   */
  public static String signString(String string) throws GeneralSecurityException, FileNotFoundException, IOException {
    byte[] data = string.getBytes("UTF8");

    KeyStore keyStore = KeyStore.getInstance(KeyStore.getDefaultType());
    keyStore.load(new FileInputStream("keystore.jks"), KEYSTORE_PASSWORD.toCharArray());

    Key key = keyStore.getKey("selfsigned", KEYSTORE_PASSWORD.toCharArray());
    if (!(key instanceof PrivateKey)) {
      fail("key is not a private key");
    }
    // Certificate keystoreCertificate = keyStore.getCertificate("selfsigned");

    Signature sig = Signature.getInstance("SHA512withRSA");
    sig.initSign((PrivateKey) key);
    sig.update(data);
    byte[] signatureBytes = sig.sign();
    String mimeCodedSignature = Base64.getEncoder().encodeToString(signatureBytes);
    // System.out.println("Singature:" + mimeCodedSignature);
    return mimeCodedSignature;
  }

  /**
   * verifies the signature of a string. The public key is stored in the local file system as X509 certificate name
   * x509.cer
   * 
   * @param string
   *          the source string
   * @param signature
   *          the signature to check
   * @return <code>true</code> if the signature is correct otherwise <code>false</code>
   * @throws UnsupportedEncodingException
   * @throws NoSuchAlgorithmException
   * @throws FileNotFoundException
   * @throws CertificateException
   * @throws InvalidKeyException
   * @throws SignatureException
   */
  public static boolean verifySignature(String string, String signature) throws UnsupportedEncodingException,
      NoSuchAlgorithmException, FileNotFoundException, CertificateException, InvalidKeyException, SignatureException {
    byte[] signatureBytes = Base64.getDecoder().decode(signature);
    byte[] data = string.getBytes("UTF8");
    Signature sig = Signature.getInstance("SHA512withRSA");

    FileInputStream fileinputstream = new FileInputStream("x509.cer");
    // Generate a certificate from the data in the file.
    CertificateFactory certificatefactory = CertificateFactory.getInstance("X.509");
    X509Certificate x509certificate = (X509Certificate) certificatefactory.generateCertificate(fileinputstream);
    sig.initVerify(x509certificate.getPublicKey());
    sig.update(data);
    return sig.verify(signatureBytes);
  }
}
